/*
 * ledapp.c -- leds manipulation *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <asm/avalanche/generic/led_config.h>
#include <asm/avalanche/generic/led_ioctl.h>

#define MAX_KEYWORDS    7
#define MAX_STATES      (MAX_MODULE_ENTRIES * MAX_STATE_ENTRIES)
#define MODULE_NAME_LEN 80


#define LED_DEV_FILE  "/dev/led"
#define CONF_FILE  "/etc/led.conf"


int read_mod(char *line);
int read_instance(char *line);
int read_state(char *line);
int read_gpio(char *line);
int read_mode(char *line);
int read_param1(char *line);
int read_param2(char *line);
int read_param(char *line);

typedef struct config_kw_handler{
	char *name;
  int (*handler)(char *line);
}config_kw_handler_t;

config_kw_handler_t kwArr[] = {
                              {"module",read_mod},
                              {"instance",read_instance},
                              {"state",read_state},
                              {"gpio",read_gpio},
                              {"mode",read_mode},
                              {"param1",read_param1},
                              {"param2",read_param2},
                              {"param",read_param}
};



char  curModule[MODULE_NAME_LEN];
int curInstance=0;
int gpio_count;
char curName[15];


LED_CONFIG_T *pTemp;
LED_CONFIG_T *ptrArr[MAX_STATES];
int ptrIndex = 0;

char arr[255];

int read_mod(char *line)
{
  /* Remember the new module's name */
  strcpy(curModule,line);
  curInstance = 0;
  return 1;
}



int read_instance(char *line)
{
  /* Remember the new instance id */
  unsigned int i;
  i = atoi(line);

  if(i < MAX_MODULE_INSTANCES)
  {
  	curInstance = i;
        return 1;
  }

  else
  {
    printf("Invalid module instance id\n");
    return 0;
  }

}




int read_state(char *line)
{

  unsigned int i;

  /* Malloc memory for new state info */
  pTemp = malloc( sizeof(LED_CONFIG_T));
  if(pTemp == NULL)
    return 0;
  /* Remember the pointer */
  ptrArr[ptrIndex++] = (LED_CONFIG_T *)pTemp;
  memset(pTemp,0x00,sizeof(LED_CONFIG_T));

  /* save the module name,instance and state var */
  strcpy(pTemp->name,curModule);
  pTemp->instance=curInstance;


  i = atoi(line);

  if(i < MAX_STATE_ENTRIES)
  { 
    pTemp->state = i;
    pTemp->gpio_num = 0;
  }

  else
  {
    printf("Invalid module state id\n");
    return 0;
  }
  return 1;
}

int read_gpio(char *line)
{
  unsigned int i;

  /* Save the LED number */
  i = atoi(line);
  
  if(i < MAX_GPIO_PIN_NUM && pTemp->gpio_num < MAX_GPIOS_PER_STATE)
  { 
     pTemp->gpio[pTemp->gpio_num++] = atoi(line);
     return 1;
  }
  else
  {
    printf("Invalid gpio id/count\n");
    return 0;
  }
}

int read_mode(char *line)
{

  unsigned int i;

  i = atoi(line);

  if(i >= NUM_LED_MODES)
  {
    printf(" invalid LED mode\n");
    return 0;
  }  

  /* Save the mode */
  if(pTemp->gpio_num >= 1)
  pTemp->mode[pTemp->gpio_num - 1] = i;
  return 1;
}

int read_param1(char *line)
{
  pTemp->param1 = atoi(line);
  return 1;
}



int read_param2(char *line)
{
  pTemp->param2 = atoi(line);
  return 1;
}


int read_param(char *line)
{
  pTemp->param1 = pTemp->param2 = atoi(line);
  return 1;
}


int read_config(char *file)
{
	FILE *in;
	char buffer[80], orig[80], *token, *line;
	int i, j, index;

	
	if (!(in = fopen(file, "r"))) {
    printf("unable to open config file\n");
		return 0;
	}
	index = -1;	
	while (fgets(buffer, 80, in)) {
		if (strchr(buffer, '\n')) *(strchr(buffer, '\n')) = '\0';
		strncpy(orig, buffer, 80);
		if (strchr(buffer, '#')) *(strchr(buffer, '#')) = '\0';
		token = buffer + strspn(buffer, " \t");
		if (*token == '\0') continue;
		line = token + strcspn(token, " \t=");
		if (*line == '\0') continue;
		*line = '\0';
		line++;
		
		/* eat leading whitespace */
		line = line + strspn(line, " \t=");
		/* eat trailing whitespace */
		for (i = strlen(line) ; i > 0 && isspace(line[i-1]); i--);
		line[i] = '\0';
    for(j=0 ; j<MAX_KEYWORDS ; j++)
    {
			if (!strcasecmp(token, kwArr[j].name))
      {
				if (!kwArr[j].handler(line))
        {
          printf("unable to parse %s\n",line);
				  return 0;
        }
      }
		}
	}
	fclose(in);
	return 1;
}

#ifdef ACTION_TEC
int safe_strcmp(const char *s1, const char *s2)
{
        if(!s1 && !s2)
                return 0;
        if(!s1)
                return -1;
        if(!s2)
                return 1;
        return strcmp(s1, s2);
}


void read_adam2_env(char* key, char* value)
{
        int res;
        char buf[4096], *ptr;
        char *pos, *pos1;
        int fd = open("/proc/ticfg/env", O_RDONLY);

        if(fd < 0)
                goto a2exit;
        res = read(fd, buf, sizeof(buf)-1);
        if(res <= 0)
                goto a2exit;
        buf[res] = 0;
        ptr = buf;
        while((pos = strchr(ptr, '\n')))
        {
                *pos = 0;
                pos++;
                pos1 = strchr(ptr, '\t');
                if(pos1)
                {
                        *pos1 = 0;
                        pos1++;
                        if(!safe_strcmp(ptr, key))
                                strncpy(value, pos1, 1024);
                }
                ptr = pos;
        }

a2exit:
        close(fd);
}
#endif

int main(int argc, char *argv[])
{
	FILE *in;
  unsigned int size;
  int i;
  LED_CONFIG_T *ptr;
  int *pBuffer;

  int fp;

#ifdef ACTION_TEC_QWEST
  {
        FILE* file = 0;
        char dslvalue[1024]={0};
        read_adam2_env("DSL_FEATURE_CNTL_0", dslvalue);

        if(!strstr(dslvalue, "0x"))
        {
            file = fopen("/proc/ticfg/env", "w");
            if(file)
            {
               fprintf(file, "DSL_FEATURE_CNTL_0 0x0020020C\n");
               fclose(file);
            }
        }

        memset (dslvalue, 0, sizeof (dslvalue));
        read_adam2_env("DSL_PHY_CNTL_0", dslvalue);
        if(!strstr(dslvalue, "0x"))
        {
            file = fopen("/proc/ticfg/env", "w");
            if(file)
            {
               fprintf(file, "DSL_PHY_CNTL_0 0x00100800\n");
               fclose(file);
           }
         }
  }
#elif defined(ACTION_TEC_VERIZON_DEMO)
  {
        FILE* file = 0;
        char dslvalue[1024]={0};
        read_adam2_env("DSL_FEATURE_CNTL_0", dslvalue);
        file = fopen("/proc/ticfg/env", "w");
        if(file)
        {
		int dsl = strtol(dslvalue,0,16);
                if( !(dsl & 0x00002000) )
                {
                        fprintf(file, "DSL_FEATURE_CNTL_0 0x00002000\n");
                        fclose(file);
                }
        }
  }
#endif

  if( argc > 3)
	{
		printf("\nLedcfg usage:   ledcfg -f <configuration file path>  \n");
		return -1;
	}
	else
  {
		if( (argc == 3) && ( strcmp( argv[1] , "-f") == 0)  && ( argv[2] != NULL) )
		{
		printf( " Reading LED config file %s \n",argv[2]);
  		read_config( argv[2] );
		}
		else if(argc == 1)
		{
		    printf( " Reading Standard Configuration File %s\n",CONF_FILE);
  		    read_config(CONF_FILE);
		}

               else
		{
		    printf("\nLedcfg usage:   ledcfg -f <configuration file path>  \n");
		    return -1;
		}


  }

   if ((fp = open(LED_DEV_FILE, O_RDWR)) == -1) {
        printf("unable to open LED device file\n");
		return 0;
	}

  /* Configure element through ioctl system calls */
  for ( i = 0 ; i < ptrIndex ; i++ )
  {

   LED_CONFIG_T *temp=ptrArr[i];

   ioctl(fp,LED_CONFIG,(int)temp);

  }


  printf("\n Configured %d states \n",ptrIndex);

  close(fp);  
  return 0;
}


