/*
  net-acct --- a network accounting system for Embedded Linux
  Copyright (C) 2004 Steven Jiang

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


  message handling of netacct

*/
#include <stdio.h>
#define __USE_GNU
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <syslog.h>
#include <errno.h>
#include "comm.h"
//#include "debug.h"
//#include "config.h"
//#include "utils.h"
//#include "net_acct.h"

int fd_msg_im;
fd_set	master_readfds;
int	max_readfd = 0;
int	input_fd;
int	input_timeout=0;

struct msg_base *message_queue = NULL;
char	msg_buffer[MAX_MESSAGE_LEN];
void	setup_input(int fd, int timeout)
{
	FD_ZERO(&master_readfds);
	FD_SET(fd, &master_readfds);
	max_readfd=fd+1;
	input_fd=fd;

	/* Set the timeout */
	input_timeout=timeout;
}

struct msg_base	*get_input( void )
{
	fd_set		readfds;
	ssize_t		recvlen;
	struct      msg_base	*msg;
	int		    active_fds;
	struct      timeval	select_tv;


	/* Copy the master fd sets into our local fd sets */
	memcpy(&readfds, &master_readfds, sizeof(fd_set));

	/* Set the timeout */
	select_tv.tv_sec = input_timeout;
	select_tv.tv_usec = 0;

	active_fds = select (max_readfd, &readfds, NULL, NULL, &select_tv);
	if(active_fds<0){
		return NULL;
	}

	/* Check for new message */
	if( FD_ISSET(input_fd, &readfds) ){
		struct sockaddr_un	from;
		socklen_t		fromlen = sizeof(struct sockaddr_un);

		recvlen = comm_recvfrom (input_fd, msg_buffer, MAX_MESSAGE_LEN -1, MSG_DONTWAIT,
				(struct sockaddr *)&from, &fromlen);
		if( recvlen > 0 && recvlen > sizeof (struct msg_base)){
			msg_buffer[recvlen]=0;
            msg = (struct msg_base *)msg_buffer;
            if (msg->datalen + sizeof (struct msg_base) < recvlen) {
                syslog (LOG_ERR, "im_verdict rcv wrong packet %d,%d", 
                        msg->datalen + sizeof (struct msg_base), recvlen);
                return NULL;
            }
            syslog (LOG_INFO, "im_verdict got seq=%d, id=%08x, len=%d",
                    msg->seq, msg->packet_id, msg->datalen);
            msg->data = msg_buffer + sizeof (struct msg_base);
			return msg;
		} else {
			/* Error */
			if(recvlen == EAGAIN){
				syslog(LOG_NOTICE, "No data after select call waiting for message\n");
			} else {
				syslog(LOG_NOTICE, "unknown error in comm_recvfrom after select call\n");
			}
		}
	}
	return NULL;
}

#include <linux/ip.h>
extern int process_packet_msg (struct iphdr *tmp_iphdr, size_t packet_length);
/*
 * The main loop of net_acct message handling
*/
int	main (void)
{
	struct msg_im_action msg_action;
	struct msg_base *msg_packet;
    int ret;
    pid_t pid;

    while( (pid = fork()) < 0) sleep(1);
    if (pid!=0) return 0;

 	if((fd_msg_im = comm_start(PC_IM_NAME, 0)) < 0 ) {
		syslog (LOG_ERR, "Error connecting to the Net Acct\n");
		return -1;
	}
    setup_input(fd_msg_im, COMM_TIMEOUT);

    syslog (LOG_INFO, "im_verdict: msg handler of im_verdict started\n");
	while (1){
		msg_packet = get_input();
        if (msg_packet) {
            msg_action.seq = msg_packet->seq;
            msg_action.action = process_packet_msg ((struct iphdr *)msg_packet->data, msg_packet->datalen);
            ret = comm_sendto(fd_msg_im, (char *)&msg_action, sizeof (struct msg_im_action), PC_IPQ_NAME);
            if (ret < 0 ) {
                syslog (LOG_ERR, "failed to send im_action");
            }
        }
	}
    exit (0);
}


