/* dhcpd.c
 *
 * udhcp DHCP client
 *
 * Russ Dill <Russ.Dill@asu.edu> July 2001
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/file.h>
#include <unistd.h>
#include <getopt.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <signal.h>
#include <time.h>
#include <string.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <errno.h>

#include "dhcpd.h"
#include "dhcpc.h"
#include "options.h"
#include "clientpacket.h"
#include "packet.h"
#include "script.h"
#include "socket.h"
#include "debug.h"
#include "pidfile.h"

#ifdef CUSTOME_HURL
	#include	<sys/socket.h>	/* basic socket definitions */
	#include	<sys/ioctl.h>
	#include <net/if.h>
#endif




static int state;
static unsigned long requested_ip; /* = 0 */
static unsigned long server_addr;
static unsigned long timeout;
static int packet_num; /* = 0 */
static int fd;
static int backup = 0;
static int failurecnt = 0;
static unsigned char backupip[16] = "0.0.0.0";

#define LISTEN_NONE 0
#define LISTEN_KERNEL 1
#define LISTEN_RAW 2
static int listen_mode;
#ifdef ACTION_TEC
	#include <stdarg.h>
	#define PPP_LED_OFF "2, 2"
	#define PPP_LED_ON "2, 1"


static void modify_dhcpd_conf(char *peerdns1, char *peerdns2,char *opt_start, char *opt_end,char *opt_gateway);
	#ifdef CUSTOME_HURL
/*
	return 1 --- True,success 
	return 0 --- False, error
 * */

		#define _PATH_TEMP      "/var/etc/hurl.XXXXXX"
		#define _MATCH_PREFIX "GOTOPAGE"
		#define _PATH_HURL_CONF "/var/etc/hurl.conf"
static const char hurl_conf[] = "GOTOPAGE %s 0\n";
static int set_gotopage(char *strp);
static int set_gotopage(char *strp)
{
	FILE *f;
	int fd;
	FILE *oldf;
//	struct stat buf;
	char pathname[128];
	char buffer[128];

	strcpy(pathname, _PATH_TEMP);
	fd = mkstemp(pathname);
	if (fd < 0) {
//		error("Unable to create temporary file");
		return 0;
	}
	f = fdopen(fd,"w");

	if (f == NULL) {
		error("Failed to open %s: %m", pathname);
//		close(fd);
		return 0;
	}
	oldf = fopen(_PATH_HURL_CONF,"r");
	if (oldf != NULL) {
		while (fgets(buffer, sizeof(buffer), oldf) != NULL) {
//			error("read %s", buffer);
			if (!strstr(buffer,_MATCH_PREFIX))
				fputs(buffer,f);
			else {
				if (strp!=NULL)
					fprintf(f, hurl_conf, strp);
			}
		}
		fclose(oldf);
		if (unlink(_PATH_HURL_CONF) < 0)
			error("unable to remove old %s: %m",_PATH_HURL_CONF);
//			return 0;
	} else
		error("Could not open %s : %m",_PATH_HURL_CONF);
//		return 0;
	/* Put the first DNS address  */

	if (ferror(f))
		error("Write failed to %s: %m", _PATH_HURL_CONF);
//		return 0;
	fclose(f);
	if (rename(pathname, _PATH_HURL_CONF) < 0)
		error("Unable to rename %s to %s: %m", pathname, _PATH_HURL_CONF);
//		return 0;

}





/*
	input:  s - is ip
			mode - 0 unset redirect, 1 set redirect
 * */
void set_hurldir(char *s,int mode);

void set_hurldir(char *s,int mode)
{
	char command[256];
	if (s==NULL)
		return;
#if 0 //just leave the code here
	if (mode)
		sprintf(&command[0],"/var/etc/hurl br0 %s add",s);
	else {
		sleep(5); //give browser time to access next page.
		sprintf(&command[0],"/var/etc/hurl br0 %s del",s);
	}
	sprintf(&command[0],"/var/etc/hurl br0 %s del",s);
#endif ACTION_TEC
    sprintf(&command[0],"/var/etc/hurl br0 %s del 5&",s);
	system(&command[0]);
}


/*input:
		WANON   - 
		WANOFF  -
*/
		#define _MATCH_PREFIXoff "WANOFF"
		#define _MATCH_PREFIXon "WANON"
int set_hurlmode(char *strp);

static const char hurl_conf1[] = "%s %s 0\n";
int set_hurlmode(char *strp)
{
	FILE *f;
	int fd;
	FILE *oldf;
//	struct stat buf;
	char pathname[128];
	char buffer[128];

	if ((strp==NULL) || ((strcmp(strp,_MATCH_PREFIXoff))&&(strcmp(strp,_MATCH_PREFIXon))))
		return 0;
	strcpy(pathname, _PATH_TEMP);
	fd = mkstemp(pathname);
	if (fd < 0) {
//		error("Unable to create temporary file");
		return 0;
	}
	f = fdopen(fd,"w");

	if (f == NULL) {
//		error("Failed to open %s: %m", pathname);
		close(fd);
		return 0;
	}
	oldf = fopen(_PATH_HURL_CONF,"r");
	if (oldf != NULL) {
		while (fgets(buffer, sizeof(buffer), oldf) != NULL) {
//			error("read %s", buffer);
			if (!strcmp(strp,_MATCH_PREFIXon)) {
				if (strstr(buffer,_MATCH_PREFIXoff)) {
					char *s;
					strtok(buffer," ");
					s=strtok(NULL," ");
					fprintf(f, hurl_conf1,strp,s);
					set_hurldir(s,0);
					continue;
				}
			} else if (!strcmp(strp,_MATCH_PREFIXoff)) {
				if (strstr(buffer,_MATCH_PREFIXon)) {
					char *s;
					strtok(buffer," ");
					s=strtok(NULL," ");
					fprintf(f, hurl_conf1,strp,s);
					set_hurldir(s,1);
					continue;
				}
			}
			fputs(buffer,f);
		}
		fclose(oldf);
		if (unlink(_PATH_HURL_CONF) < 0)
			error("unable to remove old %s: %m",_PATH_HURL_CONF);
//			return 0;
	} else
		error("Could not open %s : %m",_PATH_HURL_CONF);
//		return 0;
	/* Put the first DNS address  */

	if (ferror(f))
		error("Write failed to %s: %m", _PATH_HURL_CONF);
//		return 0;
	fclose(f);
	if (rename(pathname, _PATH_HURL_CONF) < 0)
		error("Unable to rename %s to %s: %m", pathname, _PATH_HURL_CONF);
//		return 0;
	return 1;
}

struct interface {
	struct interface *next, *prev; 
	char name[IFNAMSIZ];	/* interface name        */
	short type;			/* if type               */
	short flags;		/* various flags         */
	int metric;			/* routing metric        */
	int mtu;			/* MTU value             */
	int tx_queue_len;		/* transmit queue length */
	struct ifmap map;		/* hardware setup        */
	struct sockaddr addr;	/* IP address            */
	struct sockaddr dstaddr;	/* P-P IP address        */
	struct sockaddr broadaddr;	/* IP broadcast address  */
	struct sockaddr netmask;	/* IP network mask       */
	struct sockaddr ipxaddr_bb;	/* IPX network address   */
	struct sockaddr ipxaddr_sn;	/* IPX network address   */
	struct sockaddr ipxaddr_e3;	/* IPX network address   */
	struct sockaddr ipxaddr_e2;	/* IPX network address   */
	struct sockaddr ddpaddr;	/* Appletalk DDP address */
	struct sockaddr ecaddr;	/* Econet address        */
	int has_ip;
	int has_ipx_bb;
	int has_ipx_sn;
	int has_ipx_e3;
	int has_ipx_e2;
	int has_ax25;
	int has_ddp;
	int has_econet;
	char hwaddr[32];		/* HW address            */
	int statistics_valid;
//    struct user_net_device_stats stats;		/* statistics            */
	int keepalive;		/* keepalive value for SLIP */
	int outfill;		/* outfill value for SLIP */
};


/* Fetch the interface configuration from the kernel. */
static int if_fetch(struct interface *ife)
{
	struct ifreq ifr;
	int fd;
	char *ifname = ife->name; 


//#if HAVE_AFINET
	/* IPv4 address? */
//    fd = get_socket_for_af(AF_INET);
	fd = socket(AF_INET, SOCK_DGRAM, 0);

	if (fd >= 0) {
		strcpy(ifr.ifr_name, ifname);
		ifr.ifr_addr.sa_family = AF_INET;
		if (ioctl(fd, SIOCGIFADDR, &ifr) == 0) {
			ife->has_ip = 1;
			ife->addr = ifr.ifr_addr;
			strcpy(ifr.ifr_name, ifname);
			if (ioctl(fd, SIOCGIFDSTADDR, &ifr) < 0)
				memset(&ife->dstaddr, 0, sizeof(struct sockaddr));
			else
				ife->dstaddr = ifr.ifr_dstaddr;

			strcpy(ifr.ifr_name, ifname);
			if (ioctl(fd, SIOCGIFBRDADDR, &ifr) < 0)
				memset(&ife->broadaddr, 0, sizeof(struct sockaddr));
			else
				ife->broadaddr = ifr.ifr_broadaddr;

			strcpy(ifr.ifr_name, ifname);
			if (ioctl(fd, SIOCGIFNETMASK, &ifr) < 0)
				memset(&ife->netmask, 0, sizeof(struct sockaddr));
			else
				ife->netmask = ifr.ifr_netmask;
		} else
			ife->has_ip	= 0;
		memset(&ife->addr, 0, sizeof(struct sockaddr));
	}
//#endif


	return 0;
}

	#endif



/*
#define MAX_LMSG_LEN 255
int logger_act (const char *fmt, ...)
{
	static char tmp_text[MAX_LMSG_LEN];
	va_list ap;
	FILE *fp;
	va_start(ap, fmt);
	vsnprintf(tmp_text, MAX_LMSG_LEN-1, fmt, ap);
	tmp_text[MAX_LMSG_LEN-1] = 0;
	va_end(ap);

	fp = fopen ("/var/log/messages", "a+");
	if (fp) {
		fprintf (fp, "%s", tmp_text);
		fclose (fp);
	}
	return 0;
} */

#include <asm/avalanche/generic/led_ioctl.h>
#include <fcntl.h>

unsigned int ppp_led_handle;
unsigned int ppp_led_handle_red;
void ppp_led_init (void)
{
    int fp;
    LED_MODULE_T led_mod;
    LED_STATE_T led_state;

    /* Open /dev/led file for LED Operation */
    fp = open("/dev/led", O_RDWR);

    if(!fp) {
        return -1;
    }
    //get pppoe led handle
    led_mod.name = "pppoe";
    led_mod.instance = 0;
    ioctl(fp, LED_GET_HANDLE, &led_mod);
    ppp_led_handle = led_mod.handle;
    //get pppoered handles
    led_mod.name = "pppoered";
    led_mod.instance = 0;
    ioctl(fp, LED_GET_HANDLE, &led_mod);
    ppp_led_handle_red = led_mod.handle;

    close (fp);
}

int ppp_led_ctl_green (int ctrl_led)
{
    int fp;
    LED_STATE_T led_state;

    /* Open /dev/led file for LED Operation */
    fp = open("/dev/led", O_RDWR);

    if(!fp) {
	return -1;
    }

    led_state.handle = ppp_led_handle;
    led_state.state_id = ctrl_led;

    ioctl(fp, LED_ACTION, &led_state);

    close(fp);

    return 0;
}

int ppp_led_ctl_red (int ctrl_led)
{
    int fp;
    LED_STATE_T led_state;

    /* Open /dev/led file for LED Operation */
    fp = open("/dev/led", O_RDWR);

    if(!fp) {
        return -1;
    }

    led_state.handle = ppp_led_handle_red;
    led_state.state_id = ctrl_led;

    ioctl(fp, LED_ACTION, &led_state);

    close(fp);

    return 0;
}

#define LED_IP_UP           9 //I don't know why this define has to be defined again
int ppp_led_ipup (int ip_up)
{
    //0 - ip down, 1- ip up; 2- not showtime, 3- showtime
    //showtime is for WR project to simulate dsl showtime
    int fp;
    char buf[10];

    /* Open /dev/led file for LED Operation */
    fp = open("/dev/led", O_RDWR);

    if(!fp) {
        return -1;
    }
    memset (buf, 0, sizeof (buf));
    sprintf (buf, "%d", ip_up);

    ioctl(fp, LED_IP_UP, &buf);

    close(fp);

    return 0;
}

int ppp_led_ctl (char *ctrl_led)
{
	FILE *fproc;
	//Don't turn off now, do it when no adsl connection
	if (strcmp (ctrl_led, PPP_LED_OFF) == 0 ) {
		//ppp_led_ctl ("9, 0"); //tell led driver ip is down
        ppp_led_ipup (0);
        ppp_led_ctl_green (0);
	} else if (strcmp (ctrl_led, PPP_LED_ON) == 0 ) {
		//ppp_led_ctl ("9, 1");  //tell led driver ip is up
		//ppp_led_ctl ("11, 2"); //turn GPIO 9 off to turn to green
        ppp_led_ipup (1);
        ppp_led_ctl_red (0);
        ppp_led_ctl_green (1);
	}
#if 0 //3.3.1 led code
	/* Open /proc/led_mod/led file for LED Operation */
	fproc = fopen("/proc/led_mod/led", "w+");

	if (!fproc) {
		return -1;
	}

	fwrite(ctrl_led, 1, strlen (ctrl_led), fproc);
	fclose(fproc);
#endif //3.3.1 led code

	return 0;
}
#endif ACTION_TEC

#define DEFAULT_SCRIPT	"/usr/share/udhcpc/default.script"

struct client_config_t client_config = {
	/* Default options. */
	abort_if_no_lease: 0,
	foreground: 0,
	quit_after_lease: 0,
	interface: "eth0",
	pidfile: NULL,
	script: DEFAULT_SCRIPT,
	clientid: NULL,
	hostname: NULL,
#ifdef ACTION_TEC
	domain: NULL,    
#endif
	ifindex: 0,
	arp: "\0\0\0\0\0\0",		/* appease gcc-3.0 */
  defroute: 1 ,
};

static void print_usage(void)
{
	printf(
"Usage: udhcpcd [OPTIONS]\n\n"
"  -b, --backupip=IP               Use backup IP on failure\n"
"  -c, --clientid=CLIENTID         Client identifier\n"
"  -H, --hostname=HOSTNAME         Client hostname\n"
#ifdef ACTION_TEC
		  "  -D, --domain=DOMAIN             Domain name \n" 
#endif
"  -f, --foreground                Do not fork after getting lease\n"
"  -i, --interface=INTERFACE       Interface to use (default: eth0)\n"
"  -n, --now                       Exit with failure if lease cannot be\n"
"                                  immediately negotiated.\n"
"  -p, --pidfile=file              Store process ID of daemon in file\n"
"  -q, --quit                      Quit after obtaining lease\n"
"  -r, --request=IP                IP address to request (default: none)\n"
"  -s, --script=file               Run file at dhcp events (default:\n"
"                                  " DEFAULT_SCRIPT ")\n"
"  -v, --version                   Display version\n"
"  -x, --noroute                   Do not install default route\n"
	);
}


/* just a little helper */
static void change_mode(int new_mode)
{
	DEBUG(LOG_INFO, "entering %s listen mode",
		new_mode ? (new_mode == 1 ? "kernel" : "raw") : "none");
	close(fd);
	fd = -1;
	listen_mode = new_mode;
}


/* SIGUSR1 handler (renew) */
static void renew_requested(int sig)
{
	sig = 0;
 
	LOG(LOG_INFO, "Received SIGUSR1");
	if (state == BOUND || state == RENEWING || state == REBINDING)
  {
	    	change_mode(LISTEN_KERNEL);
		packet_num = 0;
		state = RENEW_REQUESTED;
	}
	else if (state == RELEASED) {
    packet_num = 0;
		change_mode(LISTEN_RAW);
		state = INIT_SELECTING;
	}

	/* Kill any timeouts because the user wants this to hurry along */
	timeout = 0;
}


/* SIGUSR2 handler (release) */
static void release_requested(int sig)
{
	sig = 0;
	LOG(LOG_INFO, "Received SIGUSR2");
	/* send release packet */
	if (state == BOUND || state == RENEWING || state == REBINDING) {
		send_release(server_addr, requested_ip); /* unicast */
		run_script(NULL, "deconfig");
	}

	change_mode(LISTEN_NONE);
	state = RELEASED;
	timeout = 0x7fffffff;
#ifdef ACTION_TEC	
	ppp_led_ctl (PPP_LED_OFF);
#endif ACTION_TEC	
}


/* Exit and cleanup */
static void exit_client(int retval)
{
	pidfile_delete(client_config.pidfile);
	CLOSE_LOG();
	exit(retval);
}


/* SIGTERM handler */
static void terminate(int sig)
{
	sig = 0;
	LOG(LOG_INFO, "Received SIGTERM");
	exit_client(0);
}


static void background(void)
{
	int pid_fd;
	if (client_config.quit_after_lease) {
		exit_client(0);
	} else if (!client_config.foreground) {
		pid_fd = pidfile_acquire(client_config.pidfile); /* hold lock during fork. */
		if (daemon(0, 0) == -1) {
			perror("fork");
			exit_client(1);
		}
		client_config.foreground = 1; /* Do not fork again. */
		pidfile_write_release(pid_fd);
	}
}


#ifdef COMBINED_BINARY
int udhcpc(int argc, char *argv[])
#else
int main(int argc, char *argv[])
#endif
{
	unsigned char *temp, *message;
	unsigned long t1 = 0, t2 = 0, xid = 0;
	unsigned long start = 0, lease;
	fd_set rfds;
	int retval;
	struct timeval tv;
	int c, len;
	struct dhcpMessage packet;
	struct in_addr temp_addr,temp_inet;
	int pid_fd;
	time_t now;
  char calline[50];

	static struct option options[] = {
		{"backupip",	required_argument,		0, 'b'},
		{"clientid",	required_argument,	0, 'c'},
		{"foreground",	no_argument,		0, 'f'},
		{"hostname",	required_argument,	0, 'H'},
#ifdef ACTION_TEC
		{"domain",  required_argument,  0, 'D'},
#endif
		{"help",	no_argument,		0, 'h'},
		{"interface",	required_argument,	0, 'i'},
		{"now", 	no_argument,		0, 'n'},
		{"pidfile",	required_argument,	0, 'p'},
		{"quit",	no_argument,		0, 'q'},
		{"request",	required_argument,	0, 'r'},
		{"script",	required_argument,	0, 's'},
		{"version",	no_argument,		0, 'v'},
		{"noroute",	no_argument,		0, 'x'},    
		{0, 0, 0, 0}
	};

	/* get options */
	while (1) {
		int option_index = 0;
#ifdef ACTION_TEC
		c = getopt_long(argc, argv, "b:c:fH:D:hi:np:qr:s:vx", options, &option_index);
#else		
		c = getopt_long(argc, argv, "b:c:fH:hi:np:qr:s:vx", options, &option_index);
#endif		
		if (c == -1) break;
		
		switch (c) {
    case 'b':
      strcpy(backupip,optarg);
      backup = 1;
      break;
		case 'c':
			len = strlen(optarg) > 255 ? 255 : strlen(optarg);
			if (client_config.clientid) free(client_config.clientid);
			client_config.clientid = malloc(len + 2);
			client_config.clientid[OPT_CODE] = DHCP_CLIENT_ID;
			client_config.clientid[OPT_LEN] = len;
			client_config.clientid[OPT_DATA] = '\0';
			strncpy(client_config.clientid + 3, optarg, len - 1);
			break;
		case 'f':
			client_config.foreground = 1;
			break;
		case 'H':
			len = strlen(optarg) > 255 ? 255 : strlen(optarg);
			if (client_config.hostname) free(client_config.hostname);
			client_config.hostname = malloc(len + 2);
			client_config.hostname[OPT_CODE] = DHCP_HOST_NAME;
			client_config.hostname[OPT_LEN] = len;
			strncpy(client_config.hostname + 2, optarg, len);
			break;
#ifdef ACTION_TEC
		case 'D':
			len = strlen(optarg) > 255 ? 255 : strlen(optarg);
			if (client_config.domain) free(client_config.domain);
			client_config.domain = malloc(len + 2);
			client_config.domain[OPT_CODE] = DHCP_DOMAIN_NAME;
			client_config.domain[OPT_LEN] = len;
			strncpy(client_config.domain + 2, optarg, len);
			break;
#endif
		case 'h':
			print_usage();
			return 0;
		case 'i':
			client_config.interface =  optarg;
			break;
		case 'n':
			client_config.abort_if_no_lease = 1;
			break;
		case 'p':
			client_config.pidfile = optarg;
			break;
		case 'q':
			client_config.quit_after_lease = 1;
			break;
		case 'r':
			requested_ip = inet_addr(optarg);
			break;
		case 's':
			client_config.script = optarg;
			break;
		case 'v':
			printf("udhcpcd, version %s\n\n", VERSION);
			exit_client(0);
		case 'x':
			client_config.defroute = 0;
			break;
		}
	}

	OPEN_LOG("udhcpc");
	LOG(LOG_INFO, "udhcp client (v%s) started", VERSION);

	pid_fd = pidfile_acquire(client_config.pidfile);
	pidfile_write_release(pid_fd);

	if (read_interface(client_config.interface, &client_config.ifindex, 
			   NULL, client_config.arp) < 0)
		exit_client(1);
		
	if (!client_config.clientid) {
		client_config.clientid = malloc(6 + 3);
		client_config.clientid[OPT_CODE] = DHCP_CLIENT_ID;
		client_config.clientid[OPT_LEN] = 7;
		client_config.clientid[OPT_DATA] = 1;
		memcpy(client_config.clientid + 3, client_config.arp, 6);
	}
    #ifdef ACTION_TEC
    ppp_led_init ();
    #endif //ACTION_TEC
	/* setup signal handlers */
	signal(SIGUSR1, renew_requested);
	signal(SIGUSR2, release_requested);
	signal(SIGTERM, terminate);
	
	state = INIT_SELECTING;
	run_script(NULL, "deconfig");
	change_mode(LISTEN_RAW);

	for (;;) {
		tv.tv_sec = timeout - time(0);
		tv.tv_usec = 0;
		FD_ZERO(&rfds);

		if (listen_mode != LISTEN_NONE && fd < 0) {
			if (listen_mode == LISTEN_KERNEL)
				fd = listen_socket(INADDR_ANY, CLIENT_PORT, client_config.interface);
			else
				fd = raw_socket(client_config.ifindex);
			if (fd < 0) {
				LOG(LOG_ERR, "FATAL: couldn't listen on socket, %s", sys_errlist[errno]);
				exit_client(0);
			}
		}
		if (fd >= 0) FD_SET(fd, &rfds);
		
		if (tv.tv_sec > 0) {
      
			DEBUG(LOG_INFO, "Waiting on select...\n");
			retval = select(fd + 1, &rfds, NULL, NULL, &tv);
		} else retval = 0; /* If we already timed out, fall through */

		now = time(0);
		if (retval == 0) {
			/* timeout dropped to zero */
			switch (state) {
			case INIT_SELECTING:
				if (packet_num < 3) {
					if (packet_num == 0)
						xid = random_xid();

					/* send discover packet */
					send_discover(xid, requested_ip); /* broadcast */
					
#ifdef ACTION_TEC
                    {
                        FILE * fp = fopen("var/tmp/auto_detect", "r");
                        if (fp) { //don't turn led red when auto-detecting
                            fclose(fp);
                            syslog (LOG_INFO, "don't turn led red when auto-detecting");
                        }
                        else {
			  if(!client_config.pidfile || strcmp(client_config.pidfile, "/var/run/detect_dhcp.pid") ) {
			    //don't turn LED red when cm_detect try to detect the DHCP status
        		//turn pppoe led red
        	            ppp_led_ctl (PPP_LED_OFF); //GPIO 13 0
        		    //ppp_led_ctl ("11, 1");		//GPIO 9  1
			    ppp_led_ctl_red (1);
			  }
                        }
                    }
#ifdef ACTION_TEC_VERIZON_WRA
		    ppp_led_ctl_green (1);
#endif
                    //ppp_led_ctl_red (1);
#endif ACTION_TEC
#ifdef CUSTOME_HURL
					{
					  FILE *fp;
					  char state[32];
					  state[32] = '\0'; 
					  fp = fopen("/proc/avalanche/avsar_modem_training", "r");
				          if(NULL != fp) {
					    fgets(state, 32, fp);
					    fclose(fp);                                            
					  }
					  if(strstr(state, "SHOWTIME"))
					    set_gotopage("hurl_dhcpcdiscover_send.html");
					}	
#endif

					timeout = now + ((packet_num == 2) ? 10 : 2);
					packet_num++;
				}
        else
        {
          if((backup == 1) && (failurecnt >= 2))
          {
					  LOG(LOG_INFO, "Static-Lease of %s",backupip);
            sprintf(calline,"ifconfig %s %s",client_config.interface,backupip);
            system(calline);
						exit_client(1);
          }
					else if (client_config.abort_if_no_lease)
          {
						LOG(LOG_INFO, "No lease, failing.");
						exit_client(1);
				  	}
					/* wait to try again */
          failurecnt++;
					packet_num = 0;
					timeout = now + 5;
				}
				break;
			case RENEW_REQUESTED:
			case REQUESTING:
				if (packet_num < 3)
        {
					/* send request packet */
					if (state == RENEW_REQUESTED)
						send_renew(xid, server_addr, requested_ip); /* unicast */
					else send_selecting(xid, server_addr, requested_ip); /* broadcast */
					
					timeout = now + ((packet_num == 2) ? 10 : 2);
					packet_num++;
				}
        else
        {
					/* timed out, go back to init state */
					state = INIT_SELECTING;
					timeout = now;
					packet_num = 0;
					change_mode(LISTEN_RAW);
				}
				break;
			case BOUND:
				/* Lease is starting to run out, time to enter renewing state */
				state = RENEWING;
				change_mode(LISTEN_KERNEL);
				DEBUG(LOG_INFO, "Entering renew state");
				/* fall right through */
			case RENEWING:
				/* Either set a new T1, or enter REBINDING state */
				if ((t2 - t1) <= (lease / 14400 + 1)) {
					/* timed out, enter rebinding state */
					state = REBINDING;
					timeout = now + (t2 - t1);
					DEBUG(LOG_INFO, "Entering rebinding state");
				} else {
					/* send a request packet */
					send_renew(xid, server_addr, requested_ip); /* unicast */
					
					t1 = (t2 - t1) / 2 + t1;
					timeout = t1 + start;
				}
				break;
			case REBINDING:
				/* Either set a new T2, or enter INIT state */
				if ((lease - t2) <= (lease / 14400 + 1)) {
					/* timed out, enter init state */
					state = INIT_SELECTING;
					LOG(LOG_INFO, "Lease lost, entering init state");
					run_script(NULL, "deconfig");
					timeout = now;
					packet_num = 0;
					change_mode(LISTEN_RAW);
				} else {
					/* send a request packet */
					send_renew(xid, 0, requested_ip); /* broadcast */

					t2 = (lease - t2) / 2 + t2;
					timeout = t2 + start;
				}
				break;
			case RELEASED:
				/* yah, I know, *you* say it would never happen */
				timeout = 0x7fffffff;
				break;
			}
		} else if (retval > 0 && listen_mode != LISTEN_NONE && FD_ISSET(fd, &rfds)) {
			/* a packet is ready, read it */
			
			if (listen_mode == LISTEN_KERNEL)
				len = get_packet(&packet, fd);
			else len = get_raw_packet(&packet, fd);
			
			if (len == -1 && errno != EINTR) {
				DEBUG(LOG_INFO, "error on read, %s, reopening socket", sys_errlist[errno]);
				change_mode(listen_mode); /* just close and reopen */
			}
			if (len < 0) continue;
			
			if (packet.xid != xid) {
				DEBUG(LOG_INFO, "Ignoring XID %lx (our xid is %lx)",
					(unsigned long) packet.xid, xid);
				continue;
			}
			
			if ((message = get_option(&packet, DHCP_MESSAGE_TYPE)) == NULL) {
				DEBUG(LOG_ERR, "couldnt get option from packet -- ignoring");
				continue;
			}
			
			switch (state) {
			case INIT_SELECTING:
				/* Must be a DHCPOFFER to one of our xid's */
				if (*message == DHCPOFFER) {
					if ((temp = get_option(&packet, DHCP_SERVER_ID))) {
						memcpy(&server_addr, temp, 4);
						xid = packet.xid;
						requested_ip = packet.yiaddr;
						
						/* enter requesting state */
						state = REQUESTING;
						timeout = now;
						packet_num = 0;
#if defined (ACTION_TEC)     // For Auto detect mode 
    {
        FILE* fp;  
        fp=fopen("/var/tmp/auto_detect", "r");
        if(fp){
            fclose(fp);
            unlink("/var/tmp/auto_detect");
            // no_dhcp_led = 1;
        }
    }
#endif
    
#ifdef CUSTOME_HURL
						//set_gotopage("hurl_dhcpcoffer.html");
#endif

					} else {
						DEBUG(LOG_ERR, "No server ID in message");
					}
				}
				break;
			case RENEW_REQUESTED:
			case REQUESTING:
			case RENEWING:
			case REBINDING:
				if (*message == DHCPACK) {
					if (!(temp = get_option(&packet, DHCP_LEASE_TIME))) {
						LOG(LOG_ERR, "No lease time with ACK, using 1 hour lease");
						lease = 60 * 60;
					} else {
						memcpy(&lease, temp, 4);
						lease = ntohl(lease);
					}
						
					/* enter bound state */
					t1 = lease / 2;
					
					/* little fixed point for n * .875 */
					t2 = (lease * 0x7) >> 3;
					temp_addr.s_addr = packet.yiaddr;

					if (!(temp = get_option(&packet, DHCP_SUBNET)))
                     				memset(&temp_inet.s_addr, 0x00, 4);
                 			else
                     				memcpy(&temp_inet.s_addr, temp, 4);
					LOG(LOG_INFO, "Subnet mask %s obtained", inet_ntoa(temp_inet));

          if(temp = get_option(&packet,DHCP_ROUTER))
          {
            memcpy(&temp_inet.s_addr,temp,4);
            LOG(LOG_INFO, "Router %s obtained", inet_ntoa(temp_inet));
          }

					start = now;
					timeout = t1 + start;
					requested_ip = packet.yiaddr;
          if(client_config.defroute == 1)
					run_script(&packet,
						   ((state == RENEWING || state == REBINDING) ? "renew" : "bound"));
          else
            run_script(&packet, "altbound");
					LOG(LOG_INFO, "Lease of %s obtained, lease time %ld",
						inet_ntoa(temp_addr),lease);
#ifdef ACTION_TEC      // Only The first DNS was updated in udhcpd.conf this time
                   
                   temp = get_option(&packet, DHCP_DNS_SERVER);
                   if(temp) {
                       memcpy(&temp_addr.s_addr, temp, 4);
                       modify_dhcpd_conf(inet_ntoa(temp_addr), NULL,NULL, NULL,NULL);
                   }
                   
					//turn ip led green
					ppp_led_ctl (PPP_LED_ON);

#endif ACTION_TEC
#ifdef CUSTOME_HURL
					set_gotopage("hurl_dhcpcup.html");
					set_hurlmode("WANON");
#endif

					state = BOUND;
					change_mode(LISTEN_NONE);
					//background();
					
				} else if (*message == DHCPNAK) {
					/* return to init state */
					LOG(LOG_INFO, "Received DHCP NAK");
#ifdef CUSTOME_HURL
					set_gotopage("hurl_dhcpcnak.html");
#endif
					run_script(&packet, "nak");
					if (state != REQUESTING)
						run_script(NULL, "deconfig");
					state = INIT_SELECTING;
					timeout = now;
					requested_ip = 0;
					packet_num = 0;
					change_mode(LISTEN_RAW);
					sleep(3); /* avoid excessive network traffic */
				}
				break;
			/* case BOUND, RELEASED: - ignore all packets */
			}					
		} else if (retval == -1 && errno == EINTR) {
			/* a signal was caught */
			
		} else {
			/* An error occured */
			DEBUG(LOG_ERR, "Error on select");
		}
		
	}
	return 0;
}

#ifdef ACTION_TEC
// Start of changing dhcpd conf
    #define _PATH_RESOLV_CONF "/etc/udhcpc/resolv.conf"
    #define _PATH_TEMP      "/var/etc/ppp/resolv.XXXXXX"
    #define _MATCH_PREFIX "opt dns"
    #define _PATH_UDHCPD_CONF "/var/tmp/udhcpd.conf"
static void modify_dhcpd_conf(char *peerdns1, char *peerdns2,char *opt_start, char *opt_end,char *opt_gateway) {    
    static const char udhcpd_conf[] = "opt dns %s %s\n";
    FILE *fp;
    int fd;
    FILE *oldf;
    char pathname[128];
    char buffer[128];
    char *p2;

    strcpy(pathname, _PATH_TEMP);
    fd = mkstemp(pathname);
    if(fd != -1) {
        fp = fdopen(fd, "w");
	    if (fp != NULL) {
            oldf = fopen(_PATH_UDHCPD_CONF,"r");
	        if (oldf) {
                while (fgets(buffer, sizeof(buffer), oldf) != NULL) {
                    if (!strstr(buffer,_MATCH_PREFIX)) {
                        fputs(buffer,fp);
		            }
	                else {
                        p2 = strtok(buffer, " \t");
			            p2 = strtok(NULL, " \t");  //dns
			            p2 = strtok(NULL, " \t\n");  //dns1_ip
                        if (p2 == NULL) {
                            p2 = "192.168.0.1";
                        }
                        fprintf(fp, udhcpd_conf, p2, peerdns1);
                    }
                }
                fclose(oldf);
            }
            else { // If the file could not be opened
            ;
            }
            fclose(fp);  // Now Temorary conf
        }

        if (unlink(_PATH_UDHCPD_CONF) < 0)
            LOG(LOG_INFO, "Unable to delete File!\n");
	        //    
        
        if (rename(pathname, _PATH_UDHCPD_CONF) < 0)
            LOG(LOG_INFO, "Unable to rename old %s", _PATH_UDHCPD_CONF);
			 
	}

}
// End of changing dhcpd conf
#endif ACTION_TEC


